!***************************************************************
!         map_stress.f90      ver2.0 2023/12/6 θ方向の補正
!              残留応力マップおよびひずみマップの作成
!            K. Suzuki, Niigata University
!        E=69.359 GPa, nu=0.3480  for Al 420 using Krener model
! Al (GPa): c11=106.78, c12=60.74, c44=28.21. 
! Ref: G.N. Kamm and G.A. Alers，J. Appl. Phys., 35, 327 (1964).
!   https://x-ray.jsms.jp/kroner/index.html
!   https://japan.rigaku.com/ja/techniques/residual-stress-analysis
!   変数の配列はUACJの実験(2023B1568)対応して，x:0~37,z=-5~5の配置を
!   i,j配列で②=0〜37， j=0〜11の二次元配列に変換
!***************************************************************

program map_stress

  implicit none
  
  character:: item1*50,item2*50,ans*10
  integer i, j, k, l, m,n,z,x
  real*8 :: E=69359.0, nu=0.3480  !MPa
  
  real*8 :: e_hm(0:500,0:500),e_hp(0:500,0:500)
  real*8 :: e_rm(0:500,0:500),e_rp(0:500,0:500)
  
  real*8 :: s_hm(0:500,0:500),s_hp(0:500,0:500)
  real*8 :: s_rm(0:500,0:500),s_rp(0:500,0:500)
  
  real*8 :: s_h(0:500,0:500),s_r(0:500,0:500)
  real*8 :: S1, S2, th0, pi=3.1415926
  
  th0=20.24*pi/180.0
  
  m=37; n=10


  print *,"strain_h.datとstrain_h.datを用意しましたか? (yes)"
  read(5,*) ans

  ! 周方向ひずみ strain_h.dat の読み込み
  open(unit=11,file='strain_h.dat',status='old')
  read(11,*) item1
  read(11,*) item2
  print *,item1
  print *,item2
  
  do i=0,m
    do j=0,n
      read(11,*) k,e_hm(i,j),e_hp(i,j)
      print *,i,j,e_hm(i,j),e_hp(i,j)
    end do
  end do
  close(11)
  print *, 'strain_h.datの読み込み完了'
  
  ! 半径方向ひずみ strain_r.dat の読み込み
  open(unit=11,file='strain_r.dat',status='old')
  read(11,*) item1
  read(11,*) item2
  
  do i=0,m
    do j=0,n
      read(11,*) k,e_rm(i,j),e_rp(i,j)
    print *,i,j,e_rm(i,j),e_rp(i,j)
    end do
  end do
  close(11)
  print *, 'strain_r.datの読み込み完了'
  
  
!  応力の計算
  S1= E/(1.0+nu)/cos(th0)**2
  S2= nu/((1.0+nu)*cos(th0)**2-2.0*nu)
  
  do j=0,n
    do i=0,m
      
      s_hm(i,j) = S1*(e_hm(i,j)+S2*(e_hm(i,j)+e_rm(i,j)))
      
      s_hp(i,j) = S1*(e_hp(i,j)+S2*(e_hp(i,j)+e_rp(i,j)))
      
      s_rm(i,j) = S1*(e_rm(i,j)+S2*(e_hm(i,j)+e_rm(i,j)))
      
      s_rp(i,j) = S1*(e_rp(i,j)+S2*(e_hp(i,j)+e_rp(i,j)))
      
    end do
  end do
  
  !                               残留応力の出力
  ! 周方向応力-y
  open(unit=12,file='stress_hm.dat',status='unknown')
    do j=0,n
      write(12,100) (s_hm(i,j),i=0,m)
    end do
  close(12)
  ! 周方向応力+y
  open(unit=12,file='stress_hp.dat',status='unknown')
    do j=0,n
      write(12,100) (s_hp(i,j),i=0,m)
    end do
  close(12)
  
  ! 周方向平均応力
  do j=0,n
    do i=0,m
      s_h(i,j) = (s_hm(i,j)+s_hp(i,j))/2.0
    end do
  end do
  
  open(unit=12,file='stress_h.dat',status='unknown')
    do j=0,n
      write(12,100) (s_h(i,j),i=0,m)
    end do
  close(12)
  
  
  open(unit=12,file='stress_rm.dat',status='unknown')
    do j=0,n
      write(12,100) (s_rm(i,j),i=0,m)
    end do
  close(12)
  
  open(unit=12,file='stress_rp.dat',status='unknown')
    do j=0,n
      write(12,100) (s_rp(i,j),i=0,m)
    end do
  close(12)
  
  ! 半径方向平均応力
  do j=0,n
    do i=0,m
      s_r(i,j) = (s_rm(i,j)+s_rp(i,j))/2.0
    end do
  end do
  
  open(unit=12,file='stress_r.dat',status='unknown')
    do j=0,n
      write(12,100) (s_r(i,j),i=0,m)
    end do
  close(12)
  
  print *,"応力計算結果を出力しました．"
  print *,"stress_h.dat, stress_hm.dat, stress_hp.dat"
  print *,"stress_r.dat, stress_rm.dat, stress_rp.dat"
  print *,"これらのファイルは，ImageJの Import > TextImageで表示できます．"
  
  
  !   ---------- Gnuplot用のデータファイル作成 ---------------
  !    周方向応力-y
  open(unit=12,file="gnu_s_hm.dat",status='unknown')
  write(12,*) "#  x,   z,    sgm_hm"
  do i=0,m
    do j=n,0,-1
      x=i; z=j-5
      write(12,101) x,z, s_hm(i,j)
    end do
    write(12,*)  !空白行の出力
  end do
  close(12)
  
  !    周方向応力+y
    open(unit=12,file="gnu_s_hp.dat",status='unknown')
  write(12,*) "#  x,   z,    sgm_hp"
  do i=0,m
    do j=n,0,-1
      x=i; z=j-5
      write(12,101) x,z, s_hp(i,j)
    end do
    write(12,*)  !空白行の出力
  end do
  close(12)
  
  
  ! 周方向の平均応力 (σh_m + σh_p)/2
  !    周方向応力
    open(unit=12,file="gnu_s_h.dat",status='unknown')
  write(12,*) "#  x,   z,    sgm_h"
  do i=0,m
    do j=n,0,-1
      x=i; z=j-5
      write(12,101) x,z, (s_hm(i,j) + s_hp(i,j))/2.0
    end do
    write(12,*)  !空白行の出力
  end do
  close(12)
  
  
  !    半径方向応力-y
    open(unit=12,file="gnu_s_rm.dat",status='unknown')
  write(12,*) "#  x,   z,    sgm_rm"
  do i=0,m
    do j=n,0,-1
      x=i; z=j-5
      write(12,101) x,z, s_rm(i,j)
    end do
    write(12,*)  !空白行の出力
  end do
  close(12)
  
  !    半径方向応力+y
    open(unit=12,file="gnu_s_rp.dat",status='unknown')
  write(12,*) "#  x,   z,    sgm_rp"
  do i=0,m
    do j=n,0,-1
      x=i; z=j-5
      write(12,101) x,z, s_rp(i,j)
    end do
    write(12,*)  !空白行の出力
  end do
  close(12)
  
  ! 半径方向の平均応力 (σh_m + σh_p)/2
  !    周方向応力
    open(unit=12,file="gnu_s_r.dat",status='unknown')
  write(12,*) "#  x,   z,    sgm_r"
  do i=0,m
    do j=n,0,-1
      x=i; z=j-5
      write(12,101) x,z, (s_rm(i,j) + s_r(i,j))/2.0
    end do
    write(12,*)  !空白行の出力
  end do
  close(12)
  
  
  ! gnuplotによる作図処理
  print *,"【gnuplotによる作図処理】美しいマップが描かれます．"
  print *,"●gnuplot用の.datファイルから残留応力マップを作図するには以下に従ってください．"
  print *,"gnu_*.datのファイルと同じディレクトリーにgnuplot_map.batを用意した上で，"
  print *,"ターミナルからgnuplotを起動する．"
  print *,"gnuplot> load 'gnuplot_map.bat'をキーインしてください．"
  
  
 100 format(38(2X,F7.2))
 101 format(I4,X,I4,X,F10.2)
 
end program map_stress